// la funzione main implementa un semplice simulatore per giocare a scacchi
// può essere usata per testare le altre funzioni, commentando alcune parti
// se necessario
package main

import "fmt"

// simula una partita
func main() {
	fmt.Println("semplice simulatore per gli scacchi\nsintassi mossa: r1 c1 r2 c2")
	fmt.Println("r1,r2 numeri 1..8; c1,c2 lettere a..h - premere invio alla fine)")
	fmt.Println("Ctrl-c per terminare")
	fmt.Println("per una corretta visualizzazione impostare il terminale come utf-8 e con sfondo bianco")

	s := CreaScacchiera()
	PrintScacchiera(s)
	prima := map[Colore]bool{BIANCO: true, NERO: true}
	namecol := map[Colore]string{BIANCO: "Bianco", NERO: "Nero"}
	turno := Colore(BIANCO) //inizia il bianco
	for mangiato := Tipo(NULL); mangiato != RE; {
		fmt.Printf("muove il %s: ", namecol[turno])
		c1, c2 := ReadMove()
		if src, dest, ok := Move(s, turno, c1, c2); ok {
			fmt.Printf("mosso:%s rimosso:%s\n", StringPezzo(src), StringPezzo(dest))
			mangiato = dest.tipo
			prima[turno] = false
			turno = !turno
		} else {
			fmt.Println("mossa non valida")
		}
		PrintScacchiera(s)
	}
	fmt.Printf("scacco! vince il %s\n", namecol[!turno])
}

func PrintScacchiera(chess Scacchiera) {
	fmt.Printf("%4s", "\u2502")
	for c := 'a'; c < 'a'+8; c++ { // riga 0 con le lettere per denotare le col.
		fmt.Printf("%3c  ", c)
	}
	hline := hline(5) // linea orizzontale
	fmt.Println(hline)
	for r := 8; r != 0; r-- {
		fmt.Printf("%-3d\u2502", r)
		for c := 'a'; c < 'a'+8; c++ {
			t := StringPezzo(chess[Casella{r, c}])
			fmt.Printf("%2s  \u2502", t)
		}
		fmt.Println(hline)
	}
}

// verifica se la mossa dalla casella 1 alla casella 2 di una certa scacchiera (che si assume correttamente definita)
// sia valida, basandosi su  CheckSrcDest, CheckAlfiere e CheckCavallo
// assume valide le mosse degli altri pezzi
// se la mossa è valida ritorna (src, dest, true), dove src e dest sono i pezzi sulle caselle di
// partenza e di arrivo (quest'ultimo sarà uguale a Pezzo{} se la casella di arrivo è vuota)
// ritorna (px, py, false), con px e py arbitrary, se la mossa non è valida
func ValidMove(s Scacchiera, turno Colore, c1, c2 Casella) (src, dest Pezzo, ok bool) {
	src, dest, ok = CheckSrcDest(s, turno, c1, c2)
	if ok { // casella di arrivo vuota o contenente pezzo di colore diverso
		switch src.tipo {
		case ALFIERE:
			ok = CheckAlfiere(s, c1, c2)
		case CAVALLO:
			ok = CheckCavallo(c1, c2)
		default:
			ok = true
		}

	}
	return
}

// esegue, se possibile, la mossa dala casella 1 alla casella 2 di una scacchiera
// (che si assume sia definita correttamente), in base al turno del colore specificato
// si basa sulla funzione validMove
// se la mossa è valida ritorna una tripla (src, dest, true), dove src e dest sono i pezzi
// sulle caselle di partenza (c1) e di destinazione (c2): quest'ultimo sarà Pezzo{}
// se la casella di arrivo è vuota
// se la mossa, per qualsiasi motivo, non è valida ritorna la tripla (Pezzo{}, Pezzo{}, false)
func Move(s Scacchiera, turno Colore, c1, c2 Casella) (src, dest Pezzo, ok bool) {
	if src, dest, ok = ValidMove(s, turno, c1, c2); ok { // mossa valida
		Put(s, src, c2) // dispone il pezzo src sulla casella di destinazione
		Remove(s, c1)   // svuota la casella di partenza
	}
	return
}

func hline(width int) string {
	r := "\n"
	for i := 0; i < 9*width; i++ {
		r += "\u2500"
	}
	return r
}
